from langchain_groq import ChatGroq
from functools import lru_cache

from pydantic import SecretStr
from threading import Lock


class ChatGroqFactory:
    """
    Factory class for creating and managing a singleton ChatGroq instance.
    Utilizes LRU caching for efficiency and ensures thread safety.
    """
    _lock: Lock = Lock()

    @classmethod
    @lru_cache(maxsize=1)
    def get_llm(cls, groq_token: SecretStr) -> ChatGroq:
        """
        Returns a cached singleton ChatGroq instance.

        Args:
            groq_token (SecretStr): The Groq API token.

        Returns:
            ChatGroq: A singleton instance of the ChatGroq LLM.
        """
        with cls._lock:
            return ChatGroq(
                model_name="llama-3.3-70b-versatile",
                temperature=1,
                groq_api_key=groq_token
            )

    @classmethod
    def clear_cache(cls) -> None:
        """
        Clears the cached ChatGroq instance.
        """
        cls.get_llm.cache_clear()
