from pymongo import MongoClient
from threading import Lock
from functools import lru_cache


class MongoClientFactory:
    """
    Factory class for creating and managing a MongoDB client instance.
    Implements the Singleton pattern with LRU caching to ensure a single shared client.
    """
    _lock: Lock = Lock()

    @classmethod
    @lru_cache(maxsize=1)
    def get_client(cls, uri: str = "mongodb://localhost:27017/") -> MongoClient:
        """
        Returns a cached singleton MongoClient instance.

        Args:
            uri (str): MongoDB connection URI. Defaults to localhost.

        Returns:
            MongoClient: The MongoDB client instance.
        """
        with cls._lock:
            return MongoClient(uri)

    @classmethod
    def close_client(cls) -> None:
        """
        Closes the MongoDB client connection and clears the cache.
        """
        client = cls.get_client.cache_info().hits and cls.get_client()
        if client:
            client.close()
            cls.get_client.cache_clear()
