from langchain_core.output_parsers import JsonOutputParser
from langchain_core.prompts import ChatPromptTemplate

from typing import List, Dict

from pydantic import SecretStr

from src.reviewsense_ecom.llm.chatGroq import ChatGroqFactory
from src.reviewsense_ecom.model.FeatureReviews import FeatureReviews
import re


def _create_reviews_parser() -> JsonOutputParser:
    """Create JSON parser for feature-specific reviews extraction"""
    return JsonOutputParser()


def _create_extraction_prompt() -> ChatPromptTemplate:
    """Create prompt for extracting feature-specific reviews with enhanced rules and sentiment analysis."""
    template = """Extract sentences about the given feature from the list of reviews.

    Rules:
    - Extract only parts discussing the specific feature.
    - Remove unrelated parts connected by 'and' or 'but'.
    - Keep original wording and capitalization.
    - If there is only one review, apply the same rules to extract sentences about the feature.

    Reviews: {reviews}
    Feature: {feature}

    Return only the parts discussing the specific feature and perform sentiment analysis for each extracted sentence in this JSON format:
    {{
        "feature_reviews": [
            {{ 
                "feature" : {feature}
                "sentence": "relevant sentence 1",
                "sentiment": "positive/negative/neutral"
            }},
            {{
                "feature" : {feature}
                "sentence": "relevant sentence 2",
                "sentiment": "positive/negative/neutral"
            }}
        ]
    }}
    """
    return ChatPromptTemplate.from_template(template)


class ReviewAnalysisService:
    def __init__(self):
        token: SecretStr[str] = SecretStr("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")
        self.llm = ChatGroqFactory.get_llm(token)
        self.parser = _create_reviews_parser()
        self.prompt = _create_extraction_prompt()

    def generate_feature_reviews_analysis(self, review: str, features: List[str]) -> List[Dict[str, str]]:
        """
        Extract feature-specific sentences from reviews with sentiment analysis.

        Args:
            review (str): Review text.
            features (List[str]): List of features to extract.

        Returns:
            List[Dict[str, str]]: Feature-specific sentences with sentiment analysis.
        """
        try:
            extracted_reviews = []
            sentences = re.split(r'(?<=[.!?])\s+', review)  # Split review into sentences

            for feature in features:
                feature_sentences = [s for s in sentences if feature.lower() in s.lower()]

                for sentence in feature_sentences:
                    result = self.prompt | self.llm | self.parser
                    response = result.invoke({
                        "reviews": sentence,
                        "feature": feature
                    })

                    parsed_data = FeatureReviews(**response)  # Validate and parse result
                    extracted_reviews.extend(parsed_data.feature_reviews)

            return extracted_reviews

        except Exception as e:
            print(f"Error extracting feature reviews: {e}")
            return []
